from bottle import route, response, Bottle, request, HTTPError


class CorsPluginObject():

    name = 'cors'
    api = 2

    def __init__(self, origins="*"):
        """ Constructor function

            param: bottle
            ptype: Bottle class
            param: origin
            ptype: string or array
        """
        bottle = Bottle()
        bottle.cors = True
        self.origins = origins
        self.bottle = bottle
        self._options_route()

    def apply(self, fn, context):
        def _enable_cors(*args, **kwargs):
            self.cors_headers()
            if request.method != 'OPTIONS':
                return fn(*args, **kwargs)

        return _enable_cors

    def _options_route(self):
        """ Function dedicated to add option route to the hole
            app with the route opject
        """
        route('/', method='OPTIONS', callback=self.options_function)
        route(
            '/<filepath:path>', method='OPTIONS', callback=self.options_function
        )

    def options_function(self):
        pass

    def cors_headers(self):
        """ Function dedicated to assing headers

            params: origins
            ptype: string or array of strings
        """
        response.headers['Access-Control-Allow-Origin'] = self.origins
        response.headers['Access-Control-Allow-Methods'] = '\
            GET, POST, PUT, PATCH, OPTIONS, DELETE'
        response.headers['Access-Control-Allow-Headers'] = '\
            Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token, Authorization'

    def abort(self, code=500, text='Unknown Error.'):
        """ Aborts execution and causes a HTTP error. """
        self.cors_headers()
        headers = response.headers.dict
        headerlist = response.headerlist
        for name, value in headerlist:
            headers[name] = '%s' % (value.strip())
        raise HTTPError(code, text, headers=headers)


cors_plugin_object = CorsPluginObject()

def cors_plugin(origins="*"):
    cors_plugin_object.origins = origins
    return cors_plugin_object

abort = cors_plugin_object.abort
cors_headers = cors_plugin_object.cors_headers



import setuptools, sys


with open("readme.rst", 'r') as fh:
    long_description = fh.read()


if sys.version_info < (3, 0):
     raise NotImplementedError("Sorry, you need at least Python 2.7 or Python 3.4+ to use bottle.")

setuptools.setup(
    name='bottle-cors-plugin',
    version='0.1.7',
    author="Alfonso Villalobos",
    author_email="alfonso@codepeat.com",
    license='MIT',
    description="The easiest way to use cors on bottle",
    long_description=long_description,
    long_description_content_type='text/x-rst',
    packages=setuptools.find_packages(),
    py_modules=['bottle_cors_plugin'],
    scripts=['bottle_cors_plugin.py'],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    install_requires=[
        "bottle>=0.12",
    ],
    python_requires='>=3.6'
)
